import {
  ConflictException,
  NotFoundException,
  Injectable,
  BadRequestException,
  ForbiddenException,
} from '@nestjs/common';
import { InjectRepository } from '@nestjs/typeorm';
import {
  FindOperator,
  LessThanOrEqual,
  MoreThanOrEqual,
  Repository,
} from 'typeorm';
import { HoraExcepcion } from './entity/hora-excepcion.entity';
import { InstitucionDia } from '../institucion-dia/entity/institucion-dia.entity';
import { Operador } from '../operador/entity/operador.entity';
import { InstitucionDiaService } from '../institucion-dia/institucion-dia.service';

@Injectable()
export class HoraExcepcionService {
  constructor(
    @InjectRepository(HoraExcepcion)
    private repository: Repository<HoraExcepcion>,
    private institucionDiaService: InstitucionDiaService,
  ) {}

  async create(
    admin: Operador,
    id_institucion_dia: number,
    hora_inicio: string,
    hora_fin: string,
  ) {
    const institucionDia = await this.institucionDiaService.findById(
      id_institucion_dia,
    );
    const busqueda: {
      hora_fin: FindOperator<string>;
      hora_inicio: FindOperator<string>;
      institucionDia: InstitucionDia;
    }[] = [];

    if (
      admin.institucion.id_institucion !=
      institucionDia.institucion.id_institucion
    )
      throw new ForbiddenException(
        'No puedes crear una horario sin servicio en este día porque no le pertenece a tu institución.',
      );
    if (hora_inicio > hora_fin)
      throw new BadRequestException(
        'La hora inicio no puede ser mayor que la hora fin.',
      );
    if (hora_inicio)
      busqueda.push({
        hora_fin: MoreThanOrEqual(hora_inicio),
        hora_inicio: LessThanOrEqual(hora_inicio),
        institucionDia,
      });
    if (hora_fin)
      busqueda.push({
        hora_fin: MoreThanOrEqual(hora_fin),
        hora_inicio: LessThanOrEqual(hora_fin),
        institucionDia,
      });
    return this.repository
      .findOne({ where: busqueda })
      .then((existeHoraExcepcion) => {
        if (existeHoraExcepcion) {
          if (
            existeHoraExcepcion.hora_fin === hora_fin &&
            existeHoraExcepcion.hora_inicio === hora_inicio
          )
            throw new ConflictException('Ya existe este horario sin servicio.');
          throw new BadRequestException(
            'Las horas escogídas se sobrelapan con otro horario sin servicio del mismo día.',
          );
        }
        return this.repository.save(
          this.repository.create({ institucionDia, hora_inicio, hora_fin }),
        );
      })
      .then((_) => ({
        message: 'Se creó correctamente un nuevo horario sin servicio.',
      }));
  }

  delete(admin: Operador, id_hora_excepcion: number) {
    return this.findById(id_hora_excepcion)
      .then((horaExcepcion) => {
        if (
          admin.institucion.id_institucion !=
          horaExcepcion.institucionDia.institucion.id_institucion
        )
          throw new ConflictException(
            'No puedes eliminar esta horario sin servicio porque no pertenece a tu institución.',
          );
        this.repository.delete(horaExcepcion);
      })
      .then((_) => ({
        message: 'Se eliminó correctamente este horario sin servicio.',
      }));
  }

  findAllByIdInstitucionDia(id_institucion_dia: number) {
    return this.institucionDiaService
      .findById(id_institucion_dia)
      .then((institucionDia) =>
        this.repository.find({ where: { institucionDia } }),
      );
  }

  findById(id_hora_excepcion: number) {
    return this.repository
      .findOne({
        join: {
          alias: 'he',
          innerJoinAndSelect: { id: 'he.institucionDia', i: 'id.institucion' },
        },
        where: { id_hora_excepcion },
      })
      .then((horaExcepcion) => {
        if (!horaExcepcion)
          throw new NotFoundException('No existe este id hora excepcion.');
        return horaExcepcion;
      });
  }
}
